@@ -16,7 +16,6 @@
 #include "base/gtest_prod_util.h"
 #include "base/location.h"
 #include "base/memory/raw_ptr.h"
-#include "base/memory/raw_ptr_exclusion.h"
 #include "base/memory/weak_ptr.h"
 #include "base/observer_list.h"
 #include "base/sequence_checker.h"
@@ -233,7 +232,9 @@ class BASE_EXPORT RunLoop {
     // A vector-based stack is more memory efficient than the default
     // deque-based stack as the active RunLoop stack isn't expected to ever
     // have more than a few entries.
-    using RunLoopStack = stack<RunLoop*, std::vector<RunLoop*>>;
+    using RunLoopStack =
+        stack<raw_ptr<RunLoop, VectorExperimental>,
+              std::vector<raw_ptr<RunLoop, VectorExperimental>>>;
 
     RunLoopStack active_run_loops_;
     ObserverList<RunLoop::NestingObserver>::Unchecked nesting_observers_;
@@ -305,9 +306,7 @@ class BASE_EXPORT RunLoop {
   // A cached reference of RunLoop::Delegate for the thread driven by this
   // RunLoop for quick access without using TLS (also allows access to state
   // from another sequence during Run(), ref. |sequence_checker_| below).
-  // This field is not a raw_ptr<> because it was filtered by the rewriter for:
-  // #union, #global-scope
-  RAW_PTR_EXCLUSION Delegate* const delegate_;
+  const raw_ptr<Delegate, DanglingUntriaged> delegate_;
 
   const Type type_;
 
