@@ -1,4 +1,4 @@
-// Copyright 2018 The Chromium Authors. All rights reserved.
+// Copyright 2018 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,8 +10,8 @@
 #include <type_traits>
 #include <utility>
 
-#include "base/optional.h"
 #include "base/parameter_pack.h"
+#include "third_party/abseil-cpp/absl/types/optional.h"
 
 // A bag of Traits (structs / enums / etc...) can be an elegant alternative to
 // the builder pattern and multiple default arguments for configuring things.
@@ -42,12 +42,11 @@
 //
 // DoSomethingAwesome might be defined as:
 //
-//   template <class... ArgTypes,
-//             class CheckArgumentsAreValid = std::enable_if_t<
-//                 trait_helpers::AreValidTraits<ValidTraits,
-//                                               ArgTypes...>::value>>
+//   template <class... ArgTypes>
+//     requires trait_helpers::AreValidTraits<ValidTraits, ArgTypes...>
 //   constexpr void DoSomethingAwesome(ArgTypes... args)
-//      : enable_feature_x(trait_helpers::HasTrait<EnableFeatureX>(args...)),
+//      : enable_feature_x(
+//            trait_helpers::HasTrait<EnableFeatureX, ArgTypes...>()),
 //        color(trait_helpers::GetEnum<Color, EnumTraitA::BLUE>(args...)) {}
 
 namespace base {
@@ -71,18 +70,14 @@ struct EmptyTrait {};
 // which isn't worth the complexity over ignoring EmptyTrait.
 template <typename... TraitsToExclude>
 struct Exclude {
-  template <typename T,
-            std::enable_if_t<ParameterPack<
-                TraitsToExclude...>::template HasType<T>()>* = nullptr>
-  static constexpr EmptyTrait Filter(T t) {
-    return EmptyTrait();
-  }
-
-  template <typename T,
-            std::enable_if_t<!ParameterPack<
-                TraitsToExclude...>::template HasType<T>()>* = nullptr>
-  static constexpr T Filter(T t) {
-    return t;
+  template <typename T>
+  static constexpr auto Filter(T t) {
+    if constexpr (ParameterPack<TraitsToExclude...>::template HasType<
+                      T>::value) {
+      return EmptyTrait();
+    } else {
+      return t;
+    }
   }
 };
 
@@ -111,10 +106,8 @@ struct InvalidTrait {};
 
 // Returns an object of type |TraitFilterType| constructed from |arg| if
 // compatible, or |InvalidTrait| otherwise.
-template <class TraitFilterType,
-          class ArgType,
-          class CheckArgumentIsCompatible = std::enable_if_t<
-              std::is_constructible<TraitFilterType, ArgType>::value>>
+template <class TraitFilterType, class ArgType>
+  requires std::constructible_from<TraitFilterType, ArgType>
 constexpr TraitFilterType GetTraitFromArg(CallFirstTag, ArgType arg) {
   return TraitFilterType(arg);
 }
@@ -128,10 +121,8 @@ constexpr InvalidTrait GetTraitFromArg(CallSecondTag, ArgType arg) {
 // argument in |args...|, or default constructed if none of the arguments are
 // compatible. This is the implementation of GetTraitFromArgList() with a
 // disambiguation tag.
-template <class TraitFilterType,
-          class... ArgTypes,
-          class TestCompatibleArgument = std::enable_if_t<any_of(
-              {std::is_constructible<TraitFilterType, ArgTypes>::value...})>>
+template <class TraitFilterType, class... ArgTypes>
+  requires(std::constructible_from<TraitFilterType, ArgTypes> || ...)
 constexpr TraitFilterType GetTraitFromArgListImpl(CallFirstTag,
                                                   ArgTypes... args) {
   return std::get<TraitFilterType>(std::make_tuple(
@@ -141,7 +132,7 @@ constexpr TraitFilterType GetTraitFromArgListImpl(CallFirstTag,
 template <class TraitFilterType, class... ArgTypes>
 constexpr TraitFilterType GetTraitFromArgListImpl(CallSecondTag,
                                                   ArgTypes... args) {
-  static_assert(std::is_constructible<TraitFilterType>::value,
+  static_assert(std::is_constructible_v<TraitFilterType>,
                 "The traits bag is missing a required trait.");
   return TraitFilterType();
 }
@@ -155,8 +146,7 @@ constexpr typename TraitFilterType::ValueType GetTraitFromArgList(
     ArgTypes... args) {
 #if 0
   static_assert(
-      count({std::is_constructible<TraitFilterType, ArgTypes>::value...},
-            true) <= 1,
+      count({std::is_constructible_v<TraitFilterType, ArgTypes>...}, true) <= 1,
       "The traits bag contains multiple traits of the same type.");
 #endif
   return GetTraitFromArgListImpl<TraitFilterType>(CallFirstTag(), args...);
@@ -182,11 +172,11 @@ struct EnumTraitFilter : public BasicTraitFilter<ArgType> {
 
 template <typename ArgType>
 struct OptionalEnumTraitFilter
-    : public BasicTraitFilter<ArgType, Optional<ArgType>> {
+    : public BasicTraitFilter<ArgType, absl::optional<ArgType>> {
   constexpr OptionalEnumTraitFilter()
-      : BasicTraitFilter<ArgType, Optional<ArgType>>(nullopt) {}
+      : BasicTraitFilter<ArgType, absl::optional<ArgType>>(absl::nullopt) {}
   constexpr OptionalEnumTraitFilter(ArgType arg)
-      : BasicTraitFilter<ArgType, Optional<ArgType>>(arg) {}
+      : BasicTraitFilter<ArgType, absl::optional<ArgType>>(arg) {}
 };
 
 // Tests whether multiple given argtument types are all valid traits according
@@ -199,12 +189,8 @@ struct RequiredEnumTraitFilter : public BasicTraitFilter<ArgType> {
 
 // Note EmptyTrait is always regarded as valid to support filtering.
 template <class ValidTraits, class T>
-struct IsValidTrait
-    : std::is_constructible<ValidTraits, T> {};
-
-template <class T>
-struct IsValidTrait<EmptyTrait, T>
-    : std::integral_constant<bool, true> {};
+concept IsValidTrait =
+    std::constructible_from<ValidTraits, T> || std::same_as<T, EmptyTrait>;
 
 // Tests whether a given trait type is valid or invalid by testing whether it is
 // convertible to the provided ValidTraits type. To use, define a ValidTraits
@@ -222,23 +208,8 @@ struct IsValidTrait<EmptyTrait, T>
 //   MoreValidTraits(MyOtherTrait);
 //   ...
 // };
-
-template <class... Ts >
-struct if_all;
-
-template <>
-struct if_all<>
-    : std::integral_constant<bool, true> {};
-
-template <class T, class... Ts >
-struct if_all<T, Ts...>
-    : std::conditional<T::value, if_all<Ts...>, std::integral_constant<bool, false>>::type {};
-
-
 template <class ValidTraits, class... ArgTypes>
-struct AreValidTraits
-    : std::integral_constant<bool,
-                             if_all<IsValidTrait<ValidTraits, ArgTypes>...>::value> {};
+concept AreValidTraits = (IsValidTrait<ValidTraits, ArgTypes> && ...);
 
 // Helper to make getting an enum from a trait more readable.
 template <typename Enum, typename... Args>
@@ -255,20 +226,18 @@ static constexpr Enum GetEnum(Args... args) {
 // Helper to make getting an optional enum from a trait with a default more
 // readable.
 template <typename Enum, typename... Args>
-static constexpr Optional<Enum> GetOptionalEnum(Args... args) {
+static constexpr absl::optional<Enum> GetOptionalEnum(Args... args) {
   return GetTraitFromArgList<OptionalEnumTraitFilter<Enum>>(args...);
 }
 
 // Helper to make checking for the presence of a trait more readable.
 template <typename Trait, typename... Args>
-static constexpr bool HasTrait(Args... args) {
+struct HasTrait : ParameterPack<Args...>::template HasType<Trait> {
 #if 0
-  static_assert(
-      count({std::is_constructible<Trait, Args>::value...}, true) <= 1,
-      "The traits bag contains multiple traits of the same type.");
+  static_assert(count({std::is_constructible_v<Trait, Args>...}, true) <= 1,
+                "The traits bag contains multiple traits of the same type.");
 #endif
-  return ParameterPack<Args...>::template HasType<Trait>();
-}
+};
 
 // If you need a template vararg constructor to delegate to a private
 // constructor, you may need to add this to the private constructor to ensure
