@@ -1,14 +1,19 @@
-// Copyright 2016 The Chromium Authors. All rights reserved.
+// Copyright 2016 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "cc/raster/staging_buffer_pool.h"
 
 #include <memory>
+#include <string>
+#include <utility>
 
-#include "base/bind.h"
+#include "base/containers/contains.h"
+#include "base/functional/bind.h"
+#include "base/ranges/algorithm.h"
 #include "base/strings/stringprintf.h"
-#include "base/threading/thread_task_runner_handle.h"
+#include "base/task/sequenced_task_runner.h"
+#include "base/task/single_thread_task_runner.h"
 #include "base/trace_event/memory_dump_manager.h"
 #include "cc/base/container_util.h"
 #include "components/viz/common/gpu/raster_context_provider.h"
@@ -57,8 +62,8 @@ void WaitForQueryResult(gpu::raster::RasterInterface* ri, GLuint query_id) {
     // be available in a finite amount of time.
     ri->ShallowFlushCHROMIUM();
 
-    base::PlatformThread::Sleep(base::TimeDelta::FromMilliseconds(
-        kCheckForQueryResultAvailableTickRateMs));
+    base::PlatformThread::Sleep(
+        base::Milliseconds(kCheckForQueryResultAvailableTickRateMs));
   }
 
   GLuint result = 0;
@@ -67,11 +72,12 @@ void WaitForQueryResult(gpu::raster::RasterInterface* ri, GLuint query_id) {
 
 }  // namespace
 
-StagingBuffer::StagingBuffer(const gfx::Size& size, viz::ResourceFormat format)
+StagingBuffer::StagingBuffer(const gfx::Size& size,
+                             viz::SharedImageFormat format)
     : size(size), format(format) {}
 
 StagingBuffer::~StagingBuffer() {
-  DCHECK(mailbox.IsZero());
+  DCHECK(!client_shared_image);
   DCHECK_EQ(query_id, 0u);
 }
 
@@ -81,14 +87,13 @@ void StagingBuffer::DestroyGLResources(gpu::raster::RasterInterface* ri,
     ri->DeleteQueriesEXT(1, &query_id);
     query_id = 0;
   }
-  if (!mailbox.IsZero()) {
-    sii->DestroySharedImage(sync_token, mailbox);
-    mailbox.SetZero();
+  if (client_shared_image) {
+    sii->DestroySharedImage(sync_token, std::move(client_shared_image));
   }
 }
 
 void StagingBuffer::OnMemoryDump(base::trace_event::ProcessMemoryDump* pmd,
-                                 viz::ResourceFormat format,
+                                 viz::SharedImageFormat dump_format,
                                  bool in_free_list) const {
   if (!gpu_memory_buffer)
     return;
@@ -98,8 +103,7 @@ void StagingBuffer::OnMemoryDump(base::trace_event::ProcessMemoryDump* pmd,
       base::StringPrintf("cc/one_copy/staging_memory/buffer_%p", this);
   MemoryAllocatorDump* buffer_dump = pmd->CreateAllocatorDump(buffer_dump_name);
 
-  uint64_t buffer_size_in_bytes =
-      viz::ResourceSizes::UncheckedSizeInBytes<uint64_t>(size, format);
+  uint64_t buffer_size_in_bytes = dump_format.EstimatedSizeInBytes(size);
   buffer_dump->AddScalar(MemoryAllocatorDump::kNameSize,
                          MemoryAllocatorDump::kUnitsBytes,
                          buffer_size_in_bytes);
@@ -127,15 +131,16 @@ StagingBufferPool::StagingBufferPool(
       staging_buffer_usage_in_bytes_(0),
       free_staging_buffer_usage_in_bytes_(0),
       staging_buffer_expiration_delay_(
-          base::TimeDelta::FromMilliseconds(kStagingBufferExpirationDelayMs)),
+          base::Milliseconds(kStagingBufferExpirationDelayMs)),
       reduce_memory_usage_pending_(false) {
   DCHECK(worker_context_provider_);
   base::trace_event::MemoryDumpManager::GetInstance()->RegisterDumpProvider(
-      this, "cc::StagingBufferPool", base::ThreadTaskRunnerHandle::Get());
+      this, "cc::StagingBufferPool",
+      base::SingleThreadTaskRunner::GetCurrentDefault());
 
-  memory_pressure_listener_.reset(new base::MemoryPressureListener(
-      base::BindRepeating(&StagingBufferPool::OnMemoryPressure,
-                          weak_ptr_factory_.GetWeakPtr())));
+  memory_pressure_listener_ = std::make_unique<base::MemoryPressureListener>(
+      FROM_HERE, base::BindRepeating(&StagingBufferPool::OnMemoryPressure,
+                                     weak_ptr_factory_.GetWeakPtr()));
 
   reduce_memory_usage_callback_ = base::BindRepeating(
       &StagingBufferPool::ReduceMemoryUsage, weak_ptr_factory_.GetWeakPtr());
@@ -172,7 +177,7 @@ bool StagingBufferPool::OnMemoryDump(
     base::trace_event::ProcessMemoryDump* pmd) {
   base::AutoLock lock(lock_);
 
-  if (args.level_of_detail == MemoryDumpLevelOfDetail::BACKGROUND) {
+  if (args.level_of_detail == MemoryDumpLevelOfDetail::kBackground) {
     std::string dump_name("cc/one_copy/staging_memory");
     MemoryAllocatorDump* dump = pmd->CreateAllocatorDump(dump_name);
     dump->AddScalar(MemoryAllocatorDump::kNameSize,
@@ -180,63 +185,51 @@ bool StagingBufferPool::OnMemoryDump(
                     staging_buffer_usage_in_bytes_);
   } else {
     for (const auto* buffer : buffers_) {
-      auto in_free_buffers =
-          std::find_if(free_buffers_.begin(), free_buffers_.end(),
-                       [buffer](const std::unique_ptr<StagingBuffer>& b) {
-                         return b.get() == buffer;
-                       });
-      buffer->OnMemoryDump(pmd, buffer->format,
-                           in_free_buffers != free_buffers_.end());
+      buffer->OnMemoryDump(
+          pmd, buffer->format,
+          base::Contains(free_buffers_, buffer,
+                         &std::unique_ptr<StagingBuffer>::get));
     }
   }
   return true;
 }
 
 void StagingBufferPool::AddStagingBuffer(const StagingBuffer* staging_buffer,
-                                         viz::ResourceFormat format) {
-  lock_.AssertAcquired();
-
-  DCHECK(buffers_.find(staging_buffer) == buffers_.end());
+                                         viz::SharedImageFormat format) {
+  DCHECK(!base::Contains(buffers_, staging_buffer));
   buffers_.insert(staging_buffer);
-  int buffer_usage_in_bytes = viz::ResourceSizes::UncheckedSizeInBytes<int>(
-      staging_buffer->size, format);
+  int buffer_usage_in_bytes = format.EstimatedSizeInBytes(staging_buffer->size);
   staging_buffer_usage_in_bytes_ += buffer_usage_in_bytes;
 }
 
 void StagingBufferPool::RemoveStagingBuffer(
     const StagingBuffer* staging_buffer) {
-  lock_.AssertAcquired();
-
-  DCHECK(buffers_.find(staging_buffer) != buffers_.end());
+  DCHECK(base::Contains(buffers_, staging_buffer));
   buffers_.erase(staging_buffer);
-  int buffer_usage_in_bytes = viz::ResourceSizes::UncheckedSizeInBytes<int>(
-      staging_buffer->size, staging_buffer->format);
+  int buffer_usage_in_bytes =
+      staging_buffer->format.EstimatedSizeInBytes(staging_buffer->size);
   DCHECK_GE(staging_buffer_usage_in_bytes_, buffer_usage_in_bytes);
   staging_buffer_usage_in_bytes_ -= buffer_usage_in_bytes;
 }
 
 void StagingBufferPool::MarkStagingBufferAsFree(
     const StagingBuffer* staging_buffer) {
-  lock_.AssertAcquired();
-
-  int buffer_usage_in_bytes = viz::ResourceSizes::UncheckedSizeInBytes<int>(
-      staging_buffer->size, staging_buffer->format);
+  int buffer_usage_in_bytes =
+      staging_buffer->format.EstimatedSizeInBytes(staging_buffer->size);
   free_staging_buffer_usage_in_bytes_ += buffer_usage_in_bytes;
 }
 
 void StagingBufferPool::MarkStagingBufferAsBusy(
     const StagingBuffer* staging_buffer) {
-  lock_.AssertAcquired();
-
-  int buffer_usage_in_bytes = viz::ResourceSizes::UncheckedSizeInBytes<int>(
-      staging_buffer->size, staging_buffer->format);
+  int buffer_usage_in_bytes =
+      staging_buffer->format.EstimatedSizeInBytes(staging_buffer->size);
   DCHECK_GE(free_staging_buffer_usage_in_bytes_, buffer_usage_in_bytes);
   free_staging_buffer_usage_in_bytes_ -= buffer_usage_in_bytes;
 }
 
 std::unique_ptr<StagingBuffer> StagingBufferPool::AcquireStagingBuffer(
     const gfx::Size& size,
-    viz::ResourceFormat format,
+    viz::SharedImageFormat format,
     uint64_t previous_content_id) {
   base::AutoLock lock(lock_);
 
@@ -287,11 +280,8 @@ std::unique_ptr<StagingBuffer> StagingBufferPool::AcquireStagingBuffer(
   // Find a staging buffer that allows us to perform partial raster when
   // using persistent GpuMemoryBuffers.
   if (use_partial_raster_ && previous_content_id) {
-    StagingBufferDeque::iterator it = std::find_if(
-        free_buffers_.begin(), free_buffers_.end(),
-        [previous_content_id](const std::unique_ptr<StagingBuffer>& buffer) {
-          return buffer->content_id == previous_content_id;
-        });
+    StagingBufferDeque::iterator it = base::ranges::find(
+        free_buffers_, previous_content_id, &StagingBuffer::content_id);
     if (it != free_buffers_.end()) {
       staging_buffer = std::move(*it);
       free_buffers_.erase(it);
@@ -301,8 +291,8 @@ std::unique_ptr<StagingBuffer> StagingBufferPool::AcquireStagingBuffer(
 
   // Find staging buffer of correct size and format.
   if (!staging_buffer) {
-    StagingBufferDeque::iterator it = std::find_if(
-        free_buffers_.begin(), free_buffers_.end(),
+    StagingBufferDeque::iterator it = base::ranges::find_if(
+        free_buffers_,
         [&size, format](const std::unique_ptr<StagingBuffer>& buffer) {
           return buffer->size == size && buffer->format == format;
         });
@@ -334,8 +324,6 @@ std::unique_ptr<StagingBuffer> StagingBufferPool::AcquireStagingBuffer(
 }
 
 base::TimeTicks StagingBufferPool::GetUsageTimeForLRUBuffer() {
-  lock_.AssertAcquired();
-
   if (!free_buffers_.empty())
     return free_buffers_.front()->last_usage;
 
@@ -346,8 +334,6 @@ base::TimeTicks StagingBufferPool::GetUsageTimeForLRUBuffer() {
 }
 
 void StagingBufferPool::ScheduleReduceMemoryUsage() {
-  lock_.AssertAcquired();
-
   if (reduce_memory_usage_pending_)
     return;
 
@@ -387,8 +373,6 @@ void StagingBufferPool::ReduceMemoryUsage() {
 }
 
 void StagingBufferPool::ReleaseBuffersNotUsedSince(base::TimeTicks time) {
-  lock_.AssertAcquired();
-
   {
     viz::RasterContextProvider::ScopedRasterContextLock scoped_context(
         worker_context_provider_);
